@@ -46,6 +46,10 @@ ControlType
     Selection of valid controls
 CostType
     Selection of valid penalty type
+IPOPT
+    Selection of IPOPT options
+ACADOS
+    Selection of ACADOS options
 
 # --- Managing the dynamics --- #
 ConfigureProblem
@@ -148,20 +152,31 @@ multiphase can be found in 'examples/torque_driven_ocp'. For ACADOS specific exa
 from .misc.__version__ import __version__
 from .dynamics.configure_problem import ConfigureProblem, DynamicsFcn, DynamicsList, Dynamics
 from .dynamics.dynamics_functions import DynamicsFunctions
-from .dynamics.fatigue_dynamics import FatigueList
-from .dynamics.xia_fatigue import XiaFatigue, XiaTauFatigue
+from .dynamics.fatigue.fatigue_dynamics import FatigueList
+from .dynamics.fatigue.xia_fatigue import XiaFatigue, XiaTauFatigue
+from .dynamics.fatigue.michaud_fatigue import MichaudFatigue, MichaudTauFatigue
+from .dynamics.fatigue.effort_perception import EffortPerception, TauEffortPerception
 from .dynamics.ode_solver import OdeSolver
 from .interfaces.biorbd_interface import BiorbdInterface
+from .interfaces.SolverOptions import Solver
 from .limits.constraints import ConstraintFcn, ConstraintList, Constraint
-from .limits.phase_transition import PhaseTransitionFcn, PhaseTransitionList
+from .limits.phase_transition import PhaseTransitionFcn, PhaseTransitionList, PhaseTransition
 from .limits.objective_functions import ObjectiveFcn, ObjectiveList, Objective
 from .limits.path_conditions import BoundsList, Bounds, InitialGuessList, InitialGuess, QAndQDotBounds
 from .limits.fatigue_path_conditions import FatigueBounds, FatigueInitialGuess
 from .limits.penalty_node import PenaltyNode, PenaltyNodeList
-from .misc.enums import Axis, Node, InterpolationType, PlotType, Solver, ControlType, CostType, Shooting, VariableType
+from .misc.enums import Axis, Node, InterpolationType, PlotType, ControlType, CostType, Shooting, VariableType
 from .misc.mapping import BiMappingList, BiMapping, Mapping
 from .optimization.non_linear_program import NonLinearProgram
 from .optimization.optimal_control_program import OptimalControlProgram
 from .optimization.receding_horizon_optimization import MovingHorizonEstimator, NonlinearModelPredictiveControl
+from .optimization.receding_horizon_optimization import (
+    CyclicNonlinearModelPredictiveControl,
+    CyclicMovingHorizonEstimator,
+    MultiCyclicNonlinearModelPredictiveControl,
+)
 from .optimization.parameters import ParameterList
 from .optimization.solution import Solution
+from .optimization.optimization_variable import OptimizationVariableList
+
+from .misc.casadi_expand import lt, le, gt, ge, if_else, if_else_zero
