@@ -12,7 +12,7 @@ class Objective(PenaltyOption):
     A placeholder for an objective function
     """
 
-    def __init__(self, objective: Any, custom_type: Any = None, phase: int = 0, **params: Any):
+    def __init__(self, objective: Any, custom_type: Any = None, phase: int = -1, **params: Any):
         """
         Parameters
         ----------
@@ -317,6 +317,7 @@ class ObjectiveFcn:
         MINIMIZE_TIME = (ObjectiveFunction.MayerFunction.Functions.minimize_time,)
         MINIMIZE_STATE = (PenaltyFunctionAbstract.Functions.minimize_states,)
         TRACK_STATE = (PenaltyFunctionAbstract.Functions.minimize_states,)
+        MINIMIZE_FATIGUE = (PenaltyFunctionAbstract.Functions.minimize_fatigue,)
         MINIMIZE_MARKERS = (PenaltyFunctionAbstract.Functions.minimize_markers,)
         TRACK_MARKERS = (PenaltyFunctionAbstract.Functions.minimize_markers,)
         MINIMIZE_MARKERS_VELOCITY = (PenaltyFunctionAbstract.Functions.minimize_markers_velocity,)
