@@ -5,11 +5,11 @@ import biorbd_casadi as biorbd
 import numpy as np
 from scipy import interpolate as sci_interp
 from scipy.integrate import solve_ivp
-from casadi import horzcat, DM, Function
+from casadi import vertcat, DM, Function
 from matplotlib import pyplot as plt
 
 from ..limits.path_conditions import InitialGuess, InitialGuessList
-from ..misc.enums import ControlType, CostType, Shooting, InterpolationType
+from ..misc.enums import ControlType, CostType, Shooting, InterpolationType, SolverType
 from ..misc.utils import check_version
 from ..optimization.non_linear_program import NonLinearProgram
 from ..optimization.optimization_variable import OptimizationVariableList, OptimizationVariable
@@ -47,7 +47,7 @@ class Solution:
         The unscaled constraint violation at each iteration
     inf_du: list
         The scaled dual infeasibility at each iteration
-    time_to_optimize: float
+    solver_time_to_optimize: float
         The total time to solve the program
     iterations: int
         The number of iterations that were required to solve the program
@@ -269,7 +269,7 @@ class Solution:
         self.lam_x = None
         self.inf_pr = None
         self.inf_du = None
-        self.time_to_optimize = None
+        self.solver_time_to_optimize = None
         self.real_time_to_optimize = None
         self.iterations = None
         self.status = None
@@ -288,19 +288,21 @@ class Solution:
                 The solution in a Ipopt-like dictionary
             """
 
-            self.vector = _sol["x"] if isinstance(_sol, dict) and "x" in _sol else _sol
-            self._cost = _sol["f"] if isinstance(_sol, dict) and "f" in _sol else None
-            self.constraints = _sol["g"] if isinstance(_sol, dict) and "g" in _sol else None
+            self.vector = _sol["x"]
+            if _sol["solver"] == SolverType.IPOPT:
+                self._cost = _sol["f"]
+                self.constraints = _sol["g"]
 
-            self.lam_g = _sol["lam_g"] if isinstance(_sol, dict) and "lam_g" in _sol else None
-            self.lam_p = _sol["lam_p"] if isinstance(_sol, dict) and "lam_p" in _sol else None
-            self.lam_x = _sol["lam_x"] if isinstance(_sol, dict) and "lam_x" in _sol else None
-            self.inf_pr = _sol["inf_pr"] if isinstance(_sol, dict) and "inf_pr" in _sol else None
-            self.inf_du = _sol["inf_du"] if isinstance(_sol, dict) and "inf_du" in _sol else None
-            self.time_to_optimize = _sol["time_tot"] if isinstance(_sol, dict) and "time_tot" in _sol else None
-            self.real_time_to_optimize = self.time_to_optimize
-            self.iterations = _sol["iter"] if isinstance(_sol, dict) and "iter" in _sol else None
-            self.status = _sol["status"] if isinstance(_sol, dict) and "status" in _sol else None
+                self.lam_g = _sol["lam_g"]
+                self.lam_p = _sol["lam_p"]
+                self.lam_x = _sol["lam_x"]
+                self.inf_pr = _sol["inf_pr"]
+                self.inf_du = _sol["inf_du"]
+
+            self.solver_time_to_optimize = _sol["solver_time_to_optimize"]
+            self.real_time_to_optimize = _sol["real_time_to_optimize"]
+            self.iterations = _sol["iter"]
+            self.status = _sol["status"]
 
             # Extract the data now for further use
             self._states, self._controls, self.parameters = self.ocp.v.to_dictionaries(self.vector)
@@ -434,7 +436,7 @@ class Solution:
         new.lam_x = deepcopy(self.lam_x)
         new.inf_pr = deepcopy(self.inf_pr)
         new.inf_du = deepcopy(self.inf_du)
-        new.time_to_optimize = deepcopy(self.time_to_optimize)
+        new.solver_time_to_optimize = deepcopy(self.solver_time_to_optimize)
         new.real_time_to_optimize = deepcopy(self.real_time_to_optimize)
         new.iterations = deepcopy(self.iterations)
 
@@ -508,7 +510,7 @@ class Solution:
             If the arrival value of a node should be discarded [True] or kept [False]. The value of an integrated
             arrival node and the beginning of the next one are expected to be almost equal when the problem converged
         use_scipy_integrator: bool
-            Ignore the dynamics defined by OCP and use an separate integrator provided by scipy
+            Ignore the dynamics defined by OCP and use a separate integrator provided by scipy
 
         Returns
         -------
@@ -528,6 +530,10 @@ class Solution:
                 "Shooting.MULTIPLE and keep_intermediate_points=False cannot be used simultaneously "
                 "since it would do nothing"
             )
+        if shooting_type == Shooting.SINGLE_CONTINUOUS and not continuous:
+            raise ValueError(
+                "Shooting.SINGLE_CONTINUOUS and continuous=False cannot be used simultaneously it is a contradiction"
+            )
 
         out = self.__perform_integration(shooting_type, keep_intermediate_points, continuous, use_scipy_integrator)
 
@@ -535,7 +541,7 @@ class Solution:
             if continuous:
                 out = out.interpolate(sum(out.ns) + 1)
             else:
-                out._states, _, out.phase_time, out.ns = out._merge_phases(skip_controls=True)
+                out._states, _, out.phase_time, out.ns = out._merge_phases(skip_controls=True, continuous=continuous)
                 out.is_merged = True
         out.is_integrated = True
 
@@ -547,10 +553,15 @@ class Solution:
         n_direct_collocation = sum([nlp.ode_solver.is_direct_collocation for nlp in self.ocp.nlp])
         if n_direct_collocation > 0 and not use_scipy_integrator:
             if continuous:
-                raise RuntimeError("Integration with direct collocation must be not continuous")
+                raise RuntimeError(
+                    "Integration with direct collocation must be not continuous if not use_scipy_integrator"
+                )
 
             if shooting_type != Shooting.MULTIPLE:
-                raise RuntimeError("Integration with direct collocation must using shooting_type=Shooting.MULTIPLE")
+                raise RuntimeError(
+                    "Integration with direct collocation must using shooting_type=Shooting.MULTIPLE "
+                    "if not use_scipy_integrator"
+                )
 
         # Copy the data
         out = self.copy(skip_data=True)
@@ -576,7 +587,7 @@ class Solution:
             if shooting_type == Shooting.SINGLE_CONTINUOUS:
                 if p != 0:
                     u0 = self._controls[p - 1]["all"][:, -1]
-                    val = self.ocp.phase_transitions[p - 1].function(horzcat(x0, x0), horzcat(u0, u0), params)
+                    val = self.ocp.phase_transitions[p - 1].function(vertcat(x0, x0), vertcat(u0, u0), params)
                     if val.shape[0] != x0.shape[0]:
                         raise RuntimeError(
                             f"Phase transition must have the same number of states ({val.shape[0]}) "
@@ -682,7 +693,8 @@ class Solution:
 
         if isinstance(n_frames, int):
             data_states, _, out.phase_time, out.ns = self._merge_phases(skip_controls=True)
-            t_all = [np.unique(np.concatenate(t_all))]
+            t_all = [np.concatenate((np.concatenate([_t[:-1] for _t in t_all]), [t_all[-1][-1]]))]
+
             n_frames = [n_frames]
             out.is_merged = True
         elif isinstance(n_frames, (list, tuple)) and len(n_frames) == len(self._states):
@@ -706,7 +718,7 @@ class Solution:
 
             x_interpolate = np.ndarray((n_elements, n_frames[p]))
             for j in range(n_elements):
-                s = sci_interp.splrep(t_phase, x_phase[j, time_index])
+                s = sci_interp.splrep(t_phase, x_phase[j, time_index], k=1)
                 x_interpolate[j, :] = sci_interp.splev(t_int, s)
             out._states[p]["all"] = x_interpolate
 
@@ -736,7 +748,7 @@ class Solution:
         new.is_merged = True
         return new
 
-    def _merge_phases(self, skip_states: bool = False, skip_controls: bool = False) -> tuple:
+    def _merge_phases(self, skip_states: bool = False, skip_controls: bool = False, continuous: bool = True) -> tuple:
         """
         Actually performing the phase merging
 
@@ -746,6 +758,8 @@ class Solution:
             If the merge should ignore the states
         skip_controls: bool
             If the merge should ignore the controls
+        continuous: bool
+            If the last frame of each phase should be kept [False] or discard [True]
 
         Returns
         -------
@@ -756,7 +770,7 @@ class Solution:
         if self.is_merged:
             return deepcopy(self._states), deepcopy(self._controls), deepcopy(self.phase_time), deepcopy(self.ns)
 
-        def _merge(data: list) -> Union[list, dict]:
+        def _merge(data: list, is_control: bool) -> Union[list, dict]:
             """
             Merge the phases of a states or controls data structure
 
@@ -764,6 +778,9 @@ class Solution:
             ----------
             data: list
                 The data to structure to merge the phases
+            is_control: bool
+                If the current data is a control
+
             Returns
             -------
             The data merged
@@ -783,24 +800,32 @@ class Solution:
             for i, key in enumerate(keys):
                 data_out[0][key] = np.ndarray((sizes[i], 0))
 
+            add = 0 if is_control or continuous else 1
             for p in range(len(data)):
                 d = data[p]
                 for key in d:
-                    data_out[0][key] = np.concatenate((data_out[0][key], d[key][:, : self.ns[p]]), axis=1)
-            for key in data[-1]:
-                data_out[0][key] = np.concatenate((data_out[0][key], data[-1][key][:, -1][:, np.newaxis]), axis=1)
+                    if self.ocp.nlp[p].ode_solver.is_direct_collocation and not is_control:
+                        steps = self.ocp.nlp[p].ode_solver.steps + 1
+                        data_out[0][key] = np.concatenate(
+                            (data_out[0][key], d[key][:, : self.ns[p] * steps + add]), axis=1
+                        )
+                    else:
+                        data_out[0][key] = np.concatenate((data_out[0][key], d[key][:, : self.ns[p] + add]), axis=1)
+            if add == 0:
+                for key in data[-1]:
+                    data_out[0][key] = np.concatenate((data_out[0][key], data[-1][key][:, -1][:, np.newaxis]), axis=1)
 
             return data_out
 
         if len(self._states) == 1:
             out_states = deepcopy(self._states)
         else:
-            out_states = _merge(self.states) if not skip_states and self._states else None
+            out_states = _merge(self.states, is_control=False) if not skip_states and self._states else None
 
         if len(self._controls) == 1:
             out_controls = deepcopy(self._controls)
         else:
-            out_controls = _merge(self.controls) if not skip_controls and self._controls else None
+            out_controls = _merge(self.controls, is_control=True) if not skip_controls and self._controls else None
         phase_time = [0] + [sum([self.phase_time[i + 1] for i in range(len(self.phase_time) - 1)])]
         ns = [sum(self.ns)]
 
@@ -938,18 +963,25 @@ class Solution:
             u = []
             target = []
             if nlp is not None:
-                col_x_idx = list(range(idx * steps, (idx + 1) * steps)) if penalty.integrate else [idx]
-                col_u_idx = [idx]
-                if penalty.derivative or penalty.explicit_derivative:
-                    col_x_idx.append((idx + 1) * steps)
-                    col_u_idx.append((idx + 1))
+                if penalty.transition:
+                    phase_post = (phase_idx + 1) % len(self._states)
+                    x = np.concatenate((self._states[phase_idx]["all"][:, -1], self._states[phase_post]["all"][:, 0]))
+                    u = np.concatenate(
+                        (self._controls[phase_idx]["all"][:, -1], self._controls[phase_post]["all"][:, 0])
+                    )
+                else:
+                    col_x_idx = list(range(idx * steps, (idx + 1) * steps)) if penalty.integrate else [idx]
+                    col_u_idx = [idx]
+                    if penalty.derivative or penalty.explicit_derivative:
+                        col_x_idx.append((idx + 1) * steps)
+                        col_u_idx.append((idx + 1))
 
-                x = self._states[phase_idx]["all"][:, col_x_idx]
-                u = self._controls[phase_idx]["all"][:, col_u_idx]
+                    x = self._states[phase_idx]["all"][:, col_x_idx]
+                    u = self._controls[phase_idx]["all"][:, col_u_idx]
                 target = penalty.target[:, penalty.node_idx.index(idx)] if penalty.target is not None else []
 
-            val.append(penalty.function(x, u, p))
-            val_weighted.append(penalty.weighted_function(x, u, p, penalty.weight, target, dt))
+            val.append(penalty.function_non_threaded(x, u, p))
+            val_weighted.append(penalty.weighted_function_non_threaded(x, u, p, penalty.weight, target, dt))
 
         val = np.nansum(val)
         val_weighted = np.nansum(val_weighted)
@@ -1026,7 +1058,10 @@ class Solution:
         elif cost_type == CostType.CONSTRAINTS:
             print_constraints(self.ocp, self)
         elif cost_type == CostType.ALL:
-            print(f"Solving time: {self.time_to_optimize} sec\nElapsed time: {self.real_time_to_optimize} sec")
+            print(
+                f"Solver reported time: {self.solver_time_to_optimize} sec\n"
+                f"Real time: {self.real_time_to_optimize} sec"
+            )
             self.print(CostType.OBJECTIVES)
             self.print(CostType.CONSTRAINTS)
         else:
